#!/usr/bin/env python
import setpath
import unittest
import os
from bike import testdata
from bike.testutils import *
from bike.query.getTypeOf import getTypeOf
from bike.parsing.fastparserast import Module
from bike.query.relationships import getAllClassesInHierarchy, \
     getAllDerivedClasses,\
     getRootClassesOfHierarchy, DerivedClassSearchState
from bike.parsing.newstuff import getModule


class TestGetRootClassesOfHierarchy(BRMTestCase):
    def test_getsRootClassFromDerivedClass(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class DerivedClass(BaseClass):
            pass

        """)
        rootclasses = self.helper(src,"DerivedClass")
        self.assertEqual("TheClass",rootclasses[0].name)
        self.assertEqual(len(rootclasses),1)        
        
    def test_getsRootClassFromDerivedDerivedClass(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class DerivedClass(BaseClass):
            pass
        class DerivedDerivedClass(DerivedClass):
            pass
        """)
        rootclasses = self.helper(src,"DerivedDerivedClass")
        self.assertEqual("TheClass",rootclasses[0].name)
        self.assertEqual(len(rootclasses),1)


    def test_getsRootClassFromDiamondOfClasses(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class DerivedClass(BaseClass):
            pass
        class DerivedDerivedClass(DerivedClass,BaseClass):
            pass
        """)
        rootclasses = self.helper(src,"DerivedDerivedClass")
        self.assertEqual("TheClass",rootclasses[0].name)
        self.assertEqual("TheClass",rootclasses[1].name)
        self.assertEqual(len(rootclasses),2)


    def test_getsRootClassesFromMultipleInheritance(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class DerivedClass:
            pass
        class DerivedDerivedClass(DerivedClass,BaseClass):
            pass
        """)
        rootclasses = self.helper(src,"DerivedDerivedClass")
        self.assertEqual("DerivedClass",rootclasses[0].name)
        self.assertEqual("TheClass",rootclasses[1].name)
        self.assertEqual(len(rootclasses),2)

    def test_getsRootClassesFromMultipleInheritanceWithNewStyleClass(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class DerivedClass(Object):
            pass
        class DerivedDerivedClass(DerivedClass,BaseClass):
            pass
        """)
        rootclasses = self.helper(src,"DerivedDerivedClass")
        self.assertEqual("DerivedClass",rootclasses[0].name)
        self.assertEqual("TheClass",rootclasses[1].name)
        self.assertEqual(len(rootclasses),2)


    def helper(self,src,classname):
        try:
            createPackageStructure(src,testdata.TheClass)
            classobj = getTypeOf(getModule(pkgstructureFile1),classname)
            return getRootClassesOfHierarchy(classobj)
        finally:
            removePackageStructure()


class TestGetAllDerivedClasses(BRMTestCase):
    def test_getsDerivedDerivedClass(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class DerivedClass(BaseClass):
            pass

        class DerivedDerivedClass(DerivedClass):
            def theMethod(self):
                print 'hello'
        """)
        derivedclasses = self.helper(src)
        self.assertEqual("DerivedClass",derivedclasses[0].name)
        self.assertEqual("DerivedDerivedClass",derivedclasses[1].name)
        self.assertEqual(2,len(derivedclasses))


    def test_getsDerivedClassWhenInheritanceSpecIsOverMultipleLines(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class b:
            pass

        class DerivedClass (b,
                           BaseClass):
            pass
        """)
        derivedclasses = self.helper(src)
        assert derivedclasses[0].name == "DerivedClass"
        self.assertEqual(1,len(derivedclasses))

    def helper(self,src):
        try:
            createPackageStructure(src,testdata.TheClass)
            classobj = getTypeOf(getModule(pkgstructureFile2),"TheClass")
            return getAllDerivedClasses(classobj)
        finally:
            removePackageStructure()


class TestDerivedClassSearchState(BRMTestCase):
    '''
    def test_getCandidateFilesForName_generatesMatchesAndAliases(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class b:
            pass

        class DerivedClass (b,
                           BaseClass):
            pass
        """)
        candidateFiles = self.helper(src)
        self.assertEqual([(pkgstructureFile1, ['BaseClass'])],candidateFiles)


    def test_getCandidateFilesForName_generatesMatchesAndAliases(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class b:
            pass

        class DerivedClass (b,
                           BaseClass):
            pass
        """)
        candidateFiles = self.helper(src)
        #self.assertEqual([(pkgstructureFile1, ['BaseClass'])],candidateFiles)
        assert 0


    def helper(self,src):
        try:
            createPackageStructure(src,testdata.TheClass)
            s = DerivedClassSearchState()
            return [res for res in s.getCandidateFilesForName("TheClass")]
        finally:
            removePackageStructure()
    '''

class TestGetAllClassesInHierarchy(BRMTestCase):

    def test_getsDerivedDerivedClass(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class DerivedClass(BaseClass):
            pass

        class DerivedDerivedClass(DerivedClass):
            def theMethod(self):
                print 'hello'
        """)

        try:
            createPackageStructure(src,testdata.TheClass)
            classobj = getTypeOf(getModule(pkgstructureFile2),"TheClass")
            allclasses = getAllClassesInHierarchy(getRoot(),classobj)
        finally:
            removePackageStructure()

        assert allclasses[0].name == "TheClass"
        assert allclasses[1].name == "DerivedClass"
        assert allclasses[2].name == "DerivedDerivedClass"



if __name__ == "__main__":
    unittest.main()
